/*
+----------------------------------------------------------------------+
|                    Class: CLauncherRestBackend                       |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.mrsuricate;

import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.utils.CTextUtils;

public class CLauncherRestBackend implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	public CLauncherRestConnectionManager connectionManager;
    
	private final String userId = "";
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+
    
	public CLauncherRestBackend(CLauncherRestConnectionManager connectionManager) {
		this.connectionManager = connectionManager;
		this.connectionManager.authenticate(); // this will call /auth/login and fetch the Bearer token
	}
	
	// +==============================================================+
	// | GET Methods                                                  |
	// +==============================================================+
	
	public String getScenarioLastRevision(int customerId, int scenarioId) {
		// GET api-v2/customers/553/scenarios/<id>/revisions/last/get
		
		String jsonContent = performGetQuery("customers/" + customerId + "/scenarios/" + scenarioId + "/revisions/last/get", "");
		return jsonContent;
	}

	public String getRunStatus(int customerId, int runId) {
		// api-v2/customers/553/runs/<run-id>/get
		
		String jsonContent = performGetQuery("customers/" + customerId + "/runs/" + runId + "/get", "");
		return jsonContent;
	}

/*
	public File getVideo(int testcaseRunId) {
		File videoFile = performDownloadQuery("video", "/"+testcaseRunId);
		return videoFile;
	}
*/
	
	// +==============================================================+
	// | POST Methods                                                 |
	// +==============================================================+

	public String getAllScenarios(int customerId) {
		// POST api-v2/customers/553/scenarios/get
		
		StringBuffer jsonInput = new StringBuffer();
		jsonInput.append("{");
		jsonInput.append("  \"page\": 1,");
		jsonInput.append("  \"limit\": 100,");
		jsonInput.append("  \"filter\": [");
		jsonInput.append("    {");
		jsonInput.append("      \"field\": \"deletedAt\",");
		jsonInput.append("      \"op\": \"eq\",");
		jsonInput.append("      \"value\": null");
		jsonInput.append("    }");
		jsonInput.append("  ],");
		jsonInput.append("  \"sort\": {");
		jsonInput.append("    \"field\": \"_id\",");
		jsonInput.append("    \"dir\": \"desc\"");
		jsonInput.append("  }");
		jsonInput.append("}");
		
		System.out.println("- - - -  data = " + jsonInput.toString());
		
		String jsonContent = performPostQuery("customers/" + customerId + "/scenarios/get", "", jsonInput.toString());
		return jsonContent;
	}
	
	
	public String launchScenarioLastRevision(int customerId, int scenarioId, int scenarioLastRevision, String browser) {
		// POST api-v2/customers/553/scenarios/<id>.<revision>/runs/manual/launch
		
		StringBuffer jsonInput = new StringBuffer();
		jsonInput.append("{");
		jsonInput.append("\"terminal\":\""                         + CTextUtils.escapeJsonValue(browser.toLowerCase()) + "\",");
		jsonInput.append("\"withIncident\":false,");
		jsonInput.append("\"incidentAttachmentDecisionForced\":{},");
		jsonInput.append("\"userId\":2114");
		jsonInput.append("}");
		
		System.out.println("- - - -  data = " + jsonInput.toString());
		
		String jsonContent = performPostQuery("customers/" + customerId + "/scenarios/" + scenarioId + "." + scenarioLastRevision + "/runs/manual/launch", "", jsonInput.toString());
		return jsonContent;
	}


	
	// -------------------------------------------
	// GET AND POST TEMPLATES
	// -------------------------------------------

   	private String performGetQuery(String methodName, String parameters) {
   		System.out.println(">>>>>> [REST/GET ] " + methodName + "()");
    	try {
	    	String jsonContent = connectionManager.getData("/" + methodName + parameters);
	    	
	    	if (jsonContent == null) { // return code != 200
	    		System.err.println(">>>>>> [REST/GET ] " + methodName + "() Failure!");
	    		return null;
	    	} else {
	    		System.out.println(">>>>>> [REST/GET ] " + methodName + "() success: " + jsonContent);
	    	}
	    	
	    	return jsonContent;
    	} catch (Exception e) {
    		System.err.println(">>>>>> [REST/GET ] " + methodName + "() Exception: " + e);
			return null;
		}
   	}
  	
/*
   	private File performDownloadQuery(String methodName, String parameters) {
   		System.out.println(">>>>>> [REST/GET ] " + methodName + "()");
    	try {
	    	File file = connectionManager.downloadData("/" + methodName + parameters);
	    	return file;
    	} catch (Exception e) {
    		System.err.println(">>>>>> [REST/GET ] " + methodName + "() Exception: " + e);
    		return null;
		}
   	}
*/
   	private String performPostQuery(String methodName, String parameters, String data) {
   		System.out.println(">>>>>> [REST/POST] " + methodName + "()");
    	try {
	    	String jsonContent = connectionManager.postData("/" + methodName + parameters, data);
	    	
	    	if (jsonContent == null) { // return code != 200
	    		System.err.println(">>>>>> [REST/POST] " + methodName + "() Failure!");
	    		return null;
	    	} else {
	    		System.out.println(">>>>>> [REST/POST] " + methodName + "() success: " + jsonContent);
	    	}
	    	
	    	return jsonContent;
    	} catch (Exception e) {
			System.err.println(">>>>>> [REST/POST] " + methodName + "() Exception: " + e);
			return null;
		}
   	}
    	
}

